package innerlocalclass;

/**
 * In computer science, a closure (also lexical closure or function closure) 
 * is a function or reference to a function together with a referencing environment—a 
 * table storing a reference to each of the non-local variables (also called free variables) 
 * of that function.A closure—unlike a plain function pointer—allows a 
 * function to access those non-local variables even when invoked outside of 
 * its immediate lexical scope.
 * 
 * @author hajo
 *
 */
public class ClosureFactory {
	
	static int a =  10;

    // Normally local variables (params) are gone after call
    // x, y must be final (can't change after object creation)
    public static IClosure getClosure(final int x, final int y, int j) {
        
        IClosure c = new IClosure() { // Hmm, instantiate interface..?
                                      // ...no! anonymous subclass created!

            { // Can't have ctor (anonymous class, no name)
              // So use instance initializer
              // j++; // No not final 
            }
            // Implicit create hidden x and y variables here
                     
            public int invoke() {
                return a + x + y;  // Using hidden variables (later when method finished)
            }
        };
        // x++;   // No
        j++;      // Ok, not used in inner class
        
        return c;
    }
}
