package iterator;

import java.util.ConcurrentModificationException;
import java.util.Iterator;

/**
 * An class with an iterator as an local anonymous inner class (class defined in
 * method and has no name)
 *
 * Possible to use in short for-loop
 *
 * @author hajo
 *
 */
// See also generic.iterator
public class ClassWithIterator implements Iterable<Integer> {

    private static final int DEFAULT_SIZE = 10;
    private final Integer[] elems = new Integer[DEFAULT_SIZE];

    public ClassWithIterator() {
        for (int i = 0; i < DEFAULT_SIZE - 1; i++) {
            elems[i] = i;
        }
    }

    public void add() {
        elems[DEFAULT_SIZE - 1] = 99;  // This is hard coded special case just for demo
    }

    @Override
    public Iterator<Integer> iterator() {
        // Must have final on local variables/parameters
        final int size = elems.length;
        // Iterator is an interface, so anonymous subclass to
        return new Iterator<Integer>() { // This is a local inner class
            int currentIndex = 0;

            @Override
            public boolean hasNext() {
                if (currentIndex < elems.length) {
                    return true;
                }
                return false;

            }

            @Override
            public Integer next() {
                if (size != elems.length) {
                    throw new ConcurrentModificationException("Modified");
                }
                Integer result = elems[currentIndex++];
                return result;
            }

            // Tricky to implement, you try...
            @Override
            public void remove() {
                throw new UnsupportedOperationException();
            }
        };
    }
}
