package localinner;

/**
 * Inner anonymous class 
 * Advanced example
 * 
 * @author hajo
 *
 */
public class InnerFactory {

    private int a = 10;  // Not local!  

    // Normally local variables (params) are gone after call
    // x, y must be final (can't change after object creation)
    // Else possible to write to local variables
    public IInner getInner(final int x, final int y, int j) {
        final int[] is = new int[2];
        IInner c = new IInner() { // Hmm, instantiate interface..?
            // ...no! anonymous subclass created!

            { // Can't have ctor (anonymous class, no name)
                // So use instance initializer
                // j++; // Error, not final 
            }
            // Implicit create hidden x and y variables here

            @Override
            public int sum() {
                return a + x + y;  // Can access a, x and y after method finished!!!
            }

            @Override
            public void setA(int a) {
                InnerFactory.this.a = a;  //Have reference to enclosing class
            }

            @Override
            public int getI(int index) {
                return is[index];
            }

            @Override
            public void setI(int index, int value) {
                is[index] = value;
            }

        };
        // x++;   // No, final
        j++;      // Ok, not used in inner class

        return c;   // Return anonymous object, method FINISHED all local vars should be gone!
    }

    public int getA() {
        return a;
    }
}
