package exception.bestpractices;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;

/**
 * If a method handles all of the checked exceptions that may be thrown 
 * by its implementation and the finally block throws the same 
 * exceptions as the rest of the code (which is common with java.io operations.) 
 * 
 * @author Java Best Practices
 */
public class FileHandler2 {

	public static void nestedFinally(String aFileName) {
		try {
			//If the constructor throws an exception, the finally block will NOT execute
			BufferedReader reader = new BufferedReader(
					new FileReader(aFileName));
			try {
				String line = null;
				while ((line = reader.readLine()) != null) {
					//process the line...
				}
			} finally {
				//no need to check for null
				//any exceptions thrown here will be caught by 
				//the outer catch block
				reader.close();
			}
		} catch (IOException ex) {
			//fLogger.severe("Problem occured : " + ex.getMessage());
		}
	}
}
