package generic.wildcard.bounded;

import java.io.Serializable;
import java.util.List;

/**
 * Bounded wild cards (only for parameterized types, i.e.
 * used with variables, arguments, return type)
 * 
 * @author hajo
 *
 */
public class BoundedWildCards {

	public static void main(String[] args) {
		// Ok
		List<? extends Number> elist;
		List<? super Number> slist;

		// No '&' only possible for type parameters
		//List<? super Number & Serializable> nslist;
	}

	// Generic class 
	public static class A0<T> {
		public List<? extends T> do1() {
			return null;
		}
		public void do2(List<? extends T> list) {
		}
		public List<? super T> do3() {
			return null;
		}
		public void do4(List<? super T> list) {
		}
	}
	
	// ----- Methods -----------
	public static <T> List<? extends T> doIt1(){
		return null;
	}
	public static <T> void doIt2(List<? extends T> list){
	}
	public static <T> List<? super T> doIt3(){
		return null;
	}
	public static <T> void doIt4(List<? super T> list){
	}
	
	// AND Of COURSE ..!
	public static <T extends Serializable> List<? extends Comparable<T>> 
	                                ultimatTriuph(List<? super T> list){
		return null;
	}
	
	
	// --------- The below is obviously silly ---------------
	
	/* No must have variable name, not a ? 
	public static class A1<? extends Number> {
	}

	public static class A2<? super Number> {
	}

	public static class A4<? extends Number & Serializable> {
	}*/

	// No must have a name for variable 
	//public <? extends Number> void doIt( ..?... t) {}
	//public <? super Number> void doOther(..?.. t) {}
	
	

}
