package reflection.instantiation;

import java.lang.reflect.Array;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;

/**
 * Creating objects using Class.forName() and Class<?> 
 * no 'new' anywhere
 * 
 * @author hajo
 *
 */
public class Main {

	public static void main(String[] args) {
		
	    A a = useReflexiveCreator();
	    
	    a = creatPlainClass();
		a.doItInA();

		IB b = createWithInterface();
		b.doIt();
		
		A[] as = createArray(4);
		as[0] = new A();
	}
	
	private static A useReflexiveCreator() {
        ReflexiveCreator<A> r = new ReflexiveCreator<>(A.class);
        A a = r.getIt();
        return a;
    }

    
	private static A creatPlainClass() {
		A a = null;
		try {
			// Need fully qualified name of class (and also in class path)
			Class<?> clazz = Class.forName("reflection.creation.A");
			// Must have default ctor
			a = (A) clazz.newInstance();
		} catch (ClassNotFoundException | InstantiationException
				| IllegalAccessException e) {
			e.printStackTrace();
		}
		return a;
	}

	private static IB createWithInterface() {
		IB is = null;
		try {
			// Create class object from name (String)
			Class<?> clazz = Class.forName("reflection.creation.B");
			// Must know interface here!
			Class<? extends IB> subClazz = clazz.asSubclass(IB.class);
			// Here we know constructor takes a String
			Constructor<? extends IB> ctor = subClazz
					.getConstructor(String.class);
			is = ctor.newInstance("Hoppsan");
		} catch (ClassNotFoundException | InstantiationException
				| IllegalAccessException | IllegalArgumentException
				| InvocationTargetException | NoSuchMethodException e) {
			e.printStackTrace();
		}
		return is;
	}

	private static A[] createArray(int size) {
		Class<?> clazz;
		A[] as = null;
		try {
			clazz = Class.forName("reflection.creation.A");
			as = (A[]) Array.newInstance(clazz, size);
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return as;
	}
	
	
	
	
}
