package generic.arrays;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Arrays and generic types
 *
 * NOTE : Arrays and generic's don't work well together, avoid
 *
 * @author hajo
 *
 */
public class GenericArrays {

    public static void main(String[] args) {
        rawArray();
        noParameterizedTypes();
    }

    // Mix of raw array and generic types. Exception!
    private static void rawArray() {
		// Array element type is interface List
        // i.e. array of lists
        List[] lsa = new List[10]; //Raw type

        // Every parameterized type is a subtype of the corresponding raw type
        lsa[0] = new ArrayList<String>(); //Assigning to raw type, ok
        lsa[0].add("abc"); // Warning, lsa raw type (Object)
        lsa[0].add(123); // Hmm, possible to mix types...

        List l = (List) lsa[0].get(0);
        l.clear();
    }

    private static void noParameterizedTypes() {
		// May not used parameterized types for arrays elements 
        // Can't perform runtime array store check
        // on parameterized types (no runtime type info)
        // 
		/*  
         List<String>[] lsa = new List<String>[10]; // NOT legal!
         Object o = lsa;
         Object[] oa = (Object[]) o;  // No warning
         List<Integer> li = new ArrayList<Integer>();
         li.add(new Integer(3));
         oa[1] = li;// Passes run time store check     
         String s = lsa[1].get(0);// ClassCastException  
         */
    }

    // ------- Type parameter for array ok. Taken from ArrayList ----------------------
    int size;
    Object[] elementData;

    public <T> T[] toArray(T[] a) {
        if (a.length < size) // Make a new array of a's runtime type, but my contents:
        {
            return (T[]) Arrays.copyOf(elementData, size, a.getClass());
        }
        // Warning because of type erasure, T gone.		
        System.arraycopy(elementData, 0, a, 0, size);
        if (a.length > size) {
            a[size] = null;
        }
        return a;
    }
}
