package generic.overrideoverload;

/**
 * Generic override is problematic
 * 
 * 
 * The overloading happens because the two methods have different signatures. 
 * This might come as a surprise, especially in the case of the instantiation  
 * WordBox<String> , where the two  reset methods have the same name and the 
 * same parameter type. 
 * 
 * The point is that the compiler decides whether a sub type method overrides 
 * or overloads a super type method when it compiles the generic sub type, 
 * independently of any instantiations of the generic sub type.  When the 
 * compiler compiles the declaration of the generic  WordBox<S extends CharSequence>
 * class, then there is no knowledge regarding the concrete type by which 
 * the type parameter  S might later be replaced.  Based on the declaration 
 * of the generic sub type the two reset methods have different signatures, 
 * namely reset(String) in the super type and  reset(S_extends_CharSequence) 
 * in the generic sub type. These are two completely different signatures that 
 * are not override-equivalent. Hence the compiler considers them overloading 
 * versions of each other.
 * 
 * @author Angelika Langer
 *
 */
public class GenericOverrideSubleties {

    public static void main(String[] args) {
        WordBox<String> city = new WordBox<>("Skogland");
        // Have reset(String t) in both classes so why no override?
        //city.reset("Stavanger");  // error: ambiguous 
    }

    public static class Box<T> {
        private T theThing;

        public Box(T t) {
            theThing = t;
        }

        public void reset(T t) {
            theThing = t;
        }

    }
    
    // CharSequence :> CharBuffer, Segment, String, StringBuffer, StringBuilder
    public static class WordBox<S extends CharSequence> extends Box<String> {
        public WordBox(S t) {
            super(t.toString().toLowerCase());
        }

        // Nope! this is overload
        //@Override
        public void reset(S t) {
            super.reset(t.toString().toLowerCase());
        }
    }

}
