package generic.wildcard.bounded;

import java.util.ArrayList;
import java.util.List;

/**
 *  Wild cards with upper bounds
 *  This is type safe covariance
 *
 */
public class UpperBound {

	public static void main(String[] args) {
		List<A> al = new ArrayList<>();
		al.add(new A());
		List<B> bl = new ArrayList<>();
		bl.add(new B());
		List<C> cl = new ArrayList<>();
		cl.add(new C());

		doItAny(al);
		//doItAny(bl);   // BAD

		doItAny2(bl); // Ok
		doItAny2(cl); // Ok   
	}

	// Works for List<A> ONLY!
	private static void doItAny(List<A> lst) {
		for (A a : lst) {
			a.doIt();
		}
	}

	// Works for List of A, B or C!
	private static void doItAny2(List<? extends A> lst) {
		for (A a : lst) {
			a.doIt();
		}
	}

	// -------- Types ---------------------

	public static class A {
		public void doIt() {
			System.out.println("A");
		}
	}

	public static class B extends A {

		@Override
		public void doIt() {
			System.out.println("B");
		}
	}

	public static class C extends B {

		@Override
		public void doIt() {
			System.out.println("C");
		}
	}

}
