package reflection.inspecting;

import java.lang.reflect.GenericDeclaration;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.lang.reflect.TypeVariable;

/**
 * This is just a sketch, it's possible to reflect generic information (not
 * runtime types)
 *
 * @author hajo
 *
 */
public class ReflectingGenericDump {

    public static void main(String[] args) throws InstantiationException,
            IllegalAccessException, SecurityException, NoSuchFieldException {
        Integer inums[] = {3, 6, 2, 8, 6};
        IMinMax<Integer> minMax = new ImplMinMax<>(inums);

        Class<?> clazz = minMax.getClass();
        Type[] gis = clazz.getGenericInterfaces();
        ParameterizedType pt = (ParameterizedType) gis[0];
        System.out.println("Interface is: " + pt);

        Class<?> rawType = (Class<?>) pt.getRawType();
        System.out.println("Raw type is: " + rawType);

        TypeVariable<?>[] tvars = clazz.getTypeParameters();
        GenericDeclaration gd = tvars[0].getGenericDeclaration();
        System.out.println(gd);
    }

	// -------- Types -------------------
    public interface IMinMax<T extends Comparable<T>> {

        T min();

        T max();
    }

    public static class ImplMinMax<T extends Comparable<T>> implements IMinMax<T> {

        private final T[] vals;

        public ImplMinMax(T[] o) {
            vals = o;
        }

        @Override
        public T min() {
            T v = vals[0];
            for (int i = 1; i < vals.length; i++) {
                if (vals[i].compareTo(v) < 0) {
                    v = vals[i];
                }
            }
            return v;
        }

        @Override
        public T max() {
            T v = vals[0];
            for (int i = 1; i < vals.length; i++) {
                if (vals[i].compareTo(v) > 0) {
                    v = vals[i];
                }
            }
            return v;
        }
    }

}
