package reflection.inspecting;

import java.io.Serializable;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * Using reflection to test private methods
 * (warning possible bad design)
 * 
 * @author hajo
 *
 */
public class ReflectiveDump {

	// Should be in Junit test
	public static void main(String[] args) throws IllegalArgumentException, IllegalAccessException {
		ClassWPrivateMethod c = new ClassWPrivateMethod();

		// Check  some possibilities
		reflectiveDump(c);

		// Use some to test
		// useReflectionToTest(c);

	}

	private static void reflectiveDump(ClassWPrivateMethod cwpm)
			throws IllegalArgumentException, IllegalAccessException {
		// The class object is entry to the reflection API
		@SuppressWarnings("unchecked")
		Class<ClassWPrivateMethod> clazz = (Class<ClassWPrivateMethod>) cwpm.getClass();

		// What is it?
		System.out.println("Annotation? " + clazz.isAnnotation());
		System.out.println("Array? " + clazz.isArray());
		System.out.println("Enum? " + clazz.isEnum());
		System.out.println("Instance of? " + clazz.isInstance(cwpm));
		System.out.println("Local? " + clazz.isLocalClass());
		System.out.println("Interface? " + clazz.isInterface());
		System.out.println("Anonymous? " + clazz.isAnonymousClass());
		
		// What can it do?
		Class<?>[] ifaces = clazz.getInterfaces();
		for (Class<?> iface : ifaces) {
			System.out.println("Implements ?" + iface);
		}
		
		// What does it contain?
		Constructor<?>[] ctors = clazz.getConstructors();
		for (Constructor<?> c : ctors) {
			System.out.println("Ctor used varags?" + c.isVarArgs());
		}
		Field[] fields = clazz.getDeclaredFields();
		for (Field f : fields) {
			System.out.println("Fieldnames " + f.getName());
			if (f.getName().equals("i")) {
				int i = (Integer) f.get(cwpm);
				System.out.println(i);
			}
		}
		Method[] methods = clazz.getDeclaredMethods();
		for (Method method : methods) {
			System.out.println("Methodnames " + method);
		}
	}
	
	// ------------- Types ----------------------
	
	public static class ClassWPrivateMethod implements Serializable {

		private static final long serialVersionUID = 1L;
		// public bad just for now..
		public int i = 7;
		public Integer[] is = new Integer[5];
		
		// Private but would like to test
		private void doIt(String s, Integer i) {
			System.out.println("DoIt with " + s + " " + i );
		}
		
		
		public void noWarning(){
			doIt("", 0);
		}

		public int getI() {
			return i;
		}


		public void setI(int i) {
			this.i = i;
		}

	}

	
	
	
	
}
