package edu.gu.cid.trie;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.List;

import org.junit.Test;



public class TestConnectableTrie {

	// ------- Testing non connected trie ------------

	@Test(expected = NullPointerException.class)
	public void testInsertNull() {
		ConnectableTrie t = new ConnectableTrie();
		t.insert(null);
	}

	@Test
	public void testInsertEmpty() {
		ConnectableTrie t = new ConnectableTrie();
		TrieNode c = t.insert("");
		assertTrue(c != null);
	}

	@Test
	public void testInsert() {
		ConnectableTrie t = new ConnectableTrie();
		TrieNode c = t.insert("abc");
		assertTrue(c != null);
	}
	
	@Test
	public void testInsertDuplicate() {
		ConnectableTrie t = new ConnectableTrie();
		TrieNode c = t.insert("abc");
		assertTrue(c != null);
		c = t.insert("abc");
		assertTrue(c != null);
		
	}
	

	@Test(expected = NullPointerException.class)
	public void testContainsNull() {
		ConnectableTrie t = new ConnectableTrie();
		t.contains(null);
	}

	@Test
	public void testContainsEmpty() {
		ConnectableTrie t = new ConnectableTrie();
		assertFalse(t.contains(""));
	}

	@Test
	public void testInsertAndContains() {
		ConnectableTrie t = new ConnectableTrie();
		t.insert("aaa");
		t.insert("aab");
		t.insert("abc");
		t.insert("aa");
		assertTrue(t.contains("aaa"));
		assertTrue(t.contains("aab"));
		assertTrue(t.contains("abc"));
		assertTrue(t.contains("aa"));
		
		assertFalse(t.contains("a"));
		assertFalse(t.contains("b"));
		assertFalse(t.contains("ab"));
		assertFalse(t.contains("abcxxxxx"));
	}

	@Test(expected = NullPointerException.class)
	public void testGetKeysForNull() {
		ConnectableTrie t = new ConnectableTrie();
		t.getKeys(null);
	}

	@Test
	public void testGetKeysForEmpty() {
		ConnectableTrie t = new ConnectableTrie();
		t.insert("aaa");
		t.insert("aab");
		t.insert("abc");
		// Empty string is prefix to any string
		List<String> ks = t.getKeys("");
		assertTrue(ks != null && ks.size() == 3);
	}

	@Test
	public void testGetKeys() {
		ConnectableTrie t = new ConnectableTrie();
		List<String> ks = t.getKeys("abcdefghi");
		assertTrue(ks != null && ks.size() == 0);
	}

	@Test(expected = NullPointerException.class)
	public void testGetValuesForNull() {
		ConnectableTrie t = new ConnectableTrie();
		t.getKeys(null);
	}

	@Test
	public void testGetValues() {
		ConnectableTrie t = new ConnectableTrie();
		List<String> vs = t.getKeys("abcdefghij");
		assertTrue(vs != null && vs.size() == 0);
	}

	// ----- Testing connected tries --------

	@Test
	public void testInsertKeysAndValuesAndGetAllValues() {
		ConnectableTrie k = new ConnectableTrie();
		ConnectableTrie v = new ConnectableTrie();

		TrieNode ck = k.insert("");
		TrieNode cv1 = v.insert("v1");
		TrieNode cv2 = v.insert("v2");
		TrieNode cv3 = v.insert("v3");
		/*ck.connect(cv1);
		ck.connect(cv2);
		ck.connect(cv3);
*/
		List<String> vs = k.getValues(""); // Will get all
		assertTrue(vs != null && vs.size() == 3);
	}
	
	
	@Test
	public void testInsertDuplicateKeysAndGetValues() {
		ConnectableTrie k = new ConnectableTrie();
		ConnectableTrie v = new ConnectableTrie();

		TrieNode ck = k.insert("abc");
		TrieNode cv1 = v.insert("v1");
		//ck.connect(cv1);
		
		ck = k.insert("abc");
		
		List<String> vs = k.getValues("abc"); 
		assertTrue(vs != null && vs.size() == 1);
		assertTrue(vs.get(0).equals("v1"));
	}

	@Test
	public void testInsertKeysAndCheckPrefix() {
		ConnectableTrie k = new ConnectableTrie();
		ConnectableTrie v = new ConnectableTrie();
		// The keys
		TrieNode ck1 = k.insert("aaa");
		TrieNode ck2 = k.insert("aab");
		TrieNode ck3 = k.insert("abc");
		// The values
		TrieNode cv1 = v.insert("");
		TrieNode cv2 = v.insert("");
		TrieNode cv3 = v.insert("");

		/*ck1.connect(cv1);
		ck2.connect(cv2);
		ck3.connect(cv3);
*/
		// Checking different prefix's
		List<String> ks = k.getKeys("aaa");
		assertTrue(ks.size() == 1);

		ks = k.getKeys("aa");
		assertTrue(ks.size() == 2);

		ks = k.getKeys("a");
		assertTrue(ks.size() == 3);
	}

	@Test
	public void testGetValuesForKeys() {
		ConnectableTrie k = new ConnectableTrie();
		ConnectableTrie v = new ConnectableTrie();
		// The keys
		TrieNode ck1 = k.insert("aaa");
		TrieNode ck2 = k.insert("aab");
		TrieNode ck3 = k.insert("abc");
		// The values
		TrieNode cv1 = v.insert("AAA");
		TrieNode cv2 = v.insert("AAB");
		TrieNode cv3 = v.insert("ABC");
/*
		ck1.connect(cv1);
		ck2.connect(cv2);
		ck3.connect(cv3);
*/
		// Get all keys starting with 'a'
		List<String> ks = k.getKeys("a");
		assertTrue(ks.size() == 3);

		List<String> vs = new ArrayList<>();
		for (String s : ks) {
			vs.addAll(k.getValues(s));
		}
		assertTrue(vs.contains("AAA") && vs.contains("AAB")
				&& vs.contains("ABC"));
	}

}
