package edu.gu.cid.translator.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.border.LineBorder;

import edu.gu.cid.translator.core.Language;
import edu.gu.cid.translator.ctrl.IOCtrl;
import edu.gu.cid.translator.event.Event;
import edu.gu.cid.translator.event.IEventHandler;

/**
 * Panel for alphabetical keys
 * 
 * @author hajo
 * 
 */
@SuppressWarnings("serial")
public class KeyboardPanel extends JPanel implements ActionListener,
		IEventHandler {

	// TODO This should of course be in some file
	private final static String KEYS_ENGLISH = "q w e r t y u i o p a s d f g h j k l z x c v b n m";
	private final static String KEYS_SWEDISH = "q w e r t y u i o p å a s d f g h j k l ö ä z x c v b n m";
	private final static int ROWS = 3;
	private final static int COLS = 14;
	private JButton[] buttons;

	private final IOCtrl ioCtrl;

	public KeyboardPanel(IOCtrl ioCtrl) {
		this.ioCtrl = ioCtrl;
		init();
		// TODO This is just for now to get GUI up and running
		initKeys(new Language(Language.Tag.en_US));
	}
	
	@Override
	public void actionPerformed(ActionEvent e) {
		// Action command is String on button i.e. "a" or "e"
		
	}

	@Override
	public void onEvent(Event evt) {
		
	}

	public void init() {
		GridLayout gridLayout = new GridLayout(ROWS, COLS);
		gridLayout.setHgap(2);
		gridLayout.setVgap(2);
		setLayout(gridLayout);
		setBorder(new LineBorder(Color.BLACK, 1));
		setPreferredSize(new Dimension(300, 80));
	}

	private void initKeys(Language lang) {
		if (buttons != null) {
			for (int i = 0; i < buttons.length; i++) {
				this.remove(buttons[i]);
			}
		}
		String keys = getKeys(lang);
		String[] ls = keys.split(" ");
		buttons = new JButton[ls.length];
		int i;
		for (i = 0; i < ls.length; i++) {
			JButton btn = new JButton(ls[i]);
			btn.setFont(GUIOptions.getFont());
			btn.setMargin(GUIOptions.getInsets());
			btn.addActionListener(this);
			buttons[i] = btn;
			this.add(btn, null);
		}
		validate(); // To update gui
	}

	private String getKeys(Language lang) {
		switch (lang.getTag()) {
		case en_US:
			return KEYS_ENGLISH;
		case sv_SV:
			return KEYS_SWEDISH;
		default:
			return KEYS_ENGLISH;
		}
	}
}
