	package swing.workerthread;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.concurrent.ExecutionException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.SwingWorker;

@SuppressWarnings("serial")
public class HandlingSlowMethodCall extends JFrame {
	JButton b1 = new JButton();
	JButton b2 = new JButton();
	JLabel label = new JLabel();

	// Type params, Integer =  Return type for doInBackground
    // Void = type of partial results (not used here, see API)
    private class SlowMethodWorker extends SwingWorker<Integer, Void> {
        // Run in background thread
        @Override
        protected Integer doInBackground() throws Exception {
            System.out.println(Thread.currentThread());
            return slowMethod();
        }

        // Run in event thread possible to get
        // results from doInBackground() using get()
        @Override
        protected void done(){  
            try {
                // Can't call this in listener, get() will block...
                // Here we update GUI;
                System.out.println(Thread.currentThread());
                label.setText(String.valueOf(get()));
            } catch (InterruptedException e) {
                e.printStackTrace();
            } catch (ExecutionException e) {
                e.printStackTrace();
            }
        }
        
        // More methods, see API
    };

   
	
	public HandlingSlowMethodCall() {
		this.setTitle("*** NON **** Blocking");
		this.setDefaultCloseOperation(EXIT_ON_CLOSE);
		b1.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
			    // Use a Swing worker
				SlowMethodWorker r = new SlowMethodWorker();
				r.execute();
				// Can't do this get will block
				//System.out.println(r.get());
			}
		});
		b2.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				System.out.println("Try me clicked...");
			}
		});
		this.setSize(400, 400);
		b1.setText("Very slow call");
		b2.setText("Try me ( will respond )");
		this.setLayout(new BorderLayout());
		this.getContentPane().add(b1, BorderLayout.NORTH);
		this.getContentPane().add(b2, BorderLayout.SOUTH);
		this.getContentPane().add(label, BorderLayout.CENTER);
		Font f = new Font(Font.SANS_SERIF, Font.BOLD, 48);
		label.setFont(f);
		this.setVisible(true);
	}

	private int slowMethod() {
		try {
			System.out.println("Started slow method");
			Thread.sleep(5000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		System.out.println("End slow method");
		return 999;
	}

	public static void main(String[] args) {
		System.out.println("main thread stared");
		// Because of Swing
		EventQueue.invokeLater(new Runnable() {

			public void run() {
				new HandlingSlowMethodCall().setVisible(true);
			}
		});
		System.out.println("main thread ended");

	}
}
