package classinvariants;

import java.util.Date;

/**
 * Using class invariants
 * 
 * @author hajo
 *
 * @inv this.sp >= -1
 * 		this.sp < elems.length
 * 		
 */
public class Stack {

	private Date[] elems;  // This is the representation...
	private int sp;        // ...and this  


	public Stack() {  // RI established
		final int DEFAULT = 8;
		elems = new Date[DEFAULT];
		sp = -1;
	} 

	// Mutator
	public void push(Date s) { //s is shared object, rep exposure!
		if( s == null){
			throw new IllegalArgumentException("Pushing null");
		}
		sp++;
		if (sp >= elems.length) {
			resize(); // Reestablish RI
		}
		elems[sp] = s; 
	}

	// Mutator
	public Date pop() {
		if (sp >= 0) {   // RI Holds
			Date s = elems[sp];
			elems[sp--] = null; // To garbage collector
			return s;           // No rep exp, elem doesn't belong anymore
		}
		throw new IllegalStateException("Empty stack");
	}

	// Accessor
	public Date top() {
		if (sp >= 0) {  
			return elems[sp]; // Rep exposure
		}
		throw new IllegalStateException("Empty stack");
	}
	
	// Avoid rep exposure (not part of interface)
	public Date topByValue() {
		if (sp >= 0) {  
			return new Date(elems[sp].getTime()); // Defensive copy
		}
		throw new IllegalStateException("Empty stack");
	} 
	
	// Accessor
	public int size() {
		return sp + 1;
	}
	
	private void resize() {  // elems increase RI preserved
		Date[] oldElems = elems;
		elems = new Date[2 * oldElems.length];
		System.arraycopy(oldElems, 0, elems, 0, oldElems.length); 
	}
	
	// Just for testing not part of any interface
	public int getSp(){
		return sp;
	}
}
