package freeze;

/**
 * Other possible way to make an freezable class i.e. initially mutable 
 * but later immutable 
 * 
 * 
 * @author hajo
 * 
 */
public final class Freezable {

	private String s;
	private Object o;

	// Assume must have
	public Freezable() {
	}
	
	public String getS() {
		// Possible exception if null
		return s;
	}

	public Object getO() {
		// Possible exception if null
		return o;
	}

	// Inner creator class
	public static class Creator {
		private Freezable f = new Freezable();

		public Creator setS(String s) {
			// No set method on f but we're inside Freezable
			f.s = s;
			return this;
		}

		public Creator setO(Object o) {
			f.o = o;
			return this;
		}

		public Freezable freeze() {
			if (f.o == null || f.s == null) {
				throw new IllegalArgumentException("Must initialize");
			}
			Freezable f = this.f;
			this.f = null;
			return f;
		}

	}

}
