package immutable;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Here we have a reference to a mutable object! 
 * Can't let is out! Must make a defensive copy
 * 
 * @author hajo
 * 
 */
public class DefensiveCopyOut {

	public static void main(String[] args) {
		// Bad attempt
		MyLeakingClass m = new MyLeakingClass();
		StringBuffer b = m.getData();
		b.append("OhOh");
		System.out.println(m.data); // State changed

		// Correct attempt
		MyImmutableClass i = new MyImmutableClass();
		b = m.getData();
		b.append("OhOh");
		System.out.println(i.data); // State **unchanged**
	}

	// An incorrect attempt to create an immutable class
	public static final class MyLeakingClass {
		// reference final, but object **NOT** immutable
		private final StringBuffer data = new StringBuffer();

		public MyLeakingClass() {
			data.append("abc");
		}

		// Sending out reference to mutable object
		public StringBuffer getData() {
			return data;
		}
	}

	// A correct attempt to create an immutable class
	public static final class MyImmutableClass {
		// reference final, but object **NOT** immutable
		private final StringBuffer data = new StringBuffer();

		public MyImmutableClass() {
			data.append("abc");
		}

		// Defensive copy
		public StringBuffer getData() {
			return new StringBuffer(data);
			//return data.toString();  // Also possible (must change return type)
		}
	}

	// Other correct immutable
	public static final class MyImmutableClass2 {
		// reference final, but object **NOT** immutable
		private final List<String> data = new ArrayList<>();

		public List<String> getData() {
			// String immutable but not list, so...
			return Collections.unmodifiableList(data);
			//return data;  BAD mutable list
		}
	}

}
