package immutable;

/**
 * This show that an immutable class possible can change state .. but
 * state/change never exposed to others
 * 
 * Here we calculate the "hash code" (large integer representing the object,
 * think "kind of id" (not necessarily unique) in a lazy fashion (not at object
 * creation) and store it for later use (cache)
 * 
 * @author hajo
 * 
 */
public final class LazyImmutable {

	private final String data; //String immutable

	// The hash code for the this
	private int hash; // Default to 0

	public LazyImmutable(String data) {
		this.data = data;
	}
	
	@Override  // More later on this
	public int hashCode() {  
		int h = hash;    // Do we have a value..?
		if (h == 0 && data.length() > 0) {  // ... no.
			for (int i = 0; i < data.length(); i++) {
				h = 31 * h + data.charAt(i);
			}
			hash = h;  // Save for future use.
		}
		return h;
	}
}
