package enums;



/**
 * Enum's should be used when a small number of constant 
 * values needed
 * @author hajo
 *
 */
public class UsingEnum {

	
	 public static void main(String[] args) {
		 WeekDay d = WeekDay.FRI;
		 
		 // Enum to String (toString automatically called)
		 System.out.println(d);
		 // 
		 for( WeekDay wd : WeekDay.values()){
			 System.out.println(wd);
		 }
		 
		 System.out.println(d.ordinal());
		 
		 // From String to enum
		 WeekDay d2 = WeekDay.valueOf("MON"); // Must match
		 System.out.println(d2);
	      
		 
		 // All possible choices defined in Order (using an enum) GOOD!
		 Order o = new Order();
		 o.setStatus(Order.OrderStatus.ACCEPTED);
		 
	 }
	 
	 // ------------------------------------------------------
	
	 // Simple enum 
	 public static enum WeekDay {
		 MON, TUE, WED, THU, FRI
	 }
	 

	 // Often inner enums
	 public static class Order{
		 private OrderStatus status;
		 public static enum OrderStatus {
			 ACCEPTED, REJECTED, PAID, SHIPPED;
		 }
		 
		 public void setStatus( OrderStatus status){
			 this.status = status;
		 }
		 public OrderStatus getStatus(){
			 return status;
		 }
	 }
	 
	 //-------- Enum's can implement interface (but not extend)---------
	 
	 public static interface IEnum {
		 public void doIt();
	 }
	 
	 
	 // Simple enum 
	 public static enum WeekDay2 implements IEnum {
		 MON, TUE, WED, THU, FRI;

		@Override
		public void doIt() {
			System.out.println("Enum can implement!");
		}
	 }
	  
}
