package markerinterface;

import java.io.Serializable;


/**
 * Serializability of a class is enabled by the class implementing the 
 * java.io.Serializable interface. Classes that do not implement this 
 * interface will not have any of their state serialized or deserialized. 
 * All subtypes of a serializable class are themselves serializable. 
 * The serialization interface has no methods or fields and serves 
 * only to identify the semantics of being serializable.
 * 
 * A no-arg constructor must be accessible to the subclass that is serializable.
 * 
 * 
 *  ...it is strongly recommended that all serializable classes explicitly declare
 *  serialVersionUID values, since the default serialVersionUID computation is
 *  highly sensitive to class details that may vary depending on compiler 
 *  implementations, and can thus result in unexpected InvalidClassExceptions 
 *  during deserialization. Therefore, to guarantee a consistent serialVersionUID
 *  value across different java compiler implementations, a serializable
 *  class must declare an explicit serialVersionUID value. 
 *  It is also strongly advised that explicit serialVersionUID declarations
 *  use the private modifier where possible, since such declarations apply 
 *  only to the immediately declaring class, serialVersionUID fields are not
 *  useful as inherited members.
 * 
 * @author hajo
 *
 */
public class Markerinterface {

	
	
	// If sure this will never be serialized 
	//@SuppressWarnings("serial") 
	public static class MyInterface implements Serializable {
	    
	    // NO METHODS TO IMPLEMENT!!!
		/**
		 * See above
		 */
		private static final long serialVersionUID = 403715943130392445L;
		
	}

}
