package typeconversions;

/**
 * An interface type is compatible with any type 
 * implementing the interface (i.e. sub type compatible with super)
 * 
 * VERY IMPORTANT!
 * 
 * @author hajo
 *
 */
public class CastingInterface {

	public static void main(String[] args) {
		// Safe, automatic, type casting
		// Both fulfill contract IA
		IA canALot = new A1();
		IA canLess = new A2();
		A1 a1 = new A1();
		A2 a2 = new A2();
		
		// Type safe
		canALot = canLess;
		canALot = a1;
		canALot = a2;
		
		canLess = canALot;
		canLess = a1;
		canLess = a2;
		
/*  No, none ok
		a1 = canALot;
		a1 = canLess;
		a1 = a2;
		
		a2 = canALot;
		a2 = canLess;
		a2 = a1;
*/		
		
		// Must cast, ...
		a1 = (A1) canALot;
		a1.doIt();
		// ... will work here ...(ImplA can more than IA)
		a1.doOther();

		a1 = (A1) canLess;
		a1.doIt();
		// Exception... 
		a1.doOther();
	}

	// -------------- TYPES -----------------------------
	
	public interface IA {
		public void doIt();
	}

	public static class A1 implements IA {
		@Override
		public void doIt() {
			System.out.println("Doing ImplA");
		}

		public void doOther() {
			System.out.println("Doing OTHER ImplA");
		}

	}

	public static class A2 implements IA {

		@Override
		public void doIt() {
			System.out.println("Doing ImplA");
		}
	}

}
